//---------------------------------------------------------------------------
// Copyright (C) 1999 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  owatod.c - This utility uses TMEX to read the DS2450 1-Wire AtoD
//
//  Compiler:  Microsoft Visual C 
//  Version:   1.00
//

#define TMEXUTIL

#include "ibtmex.h"   
#include "owatod.h"

// external function prototypes 
extern int RunScript(Script script[], void *, int);

// local function prototypes
int PrintHeader(char *);
void output_status(int, char *);

// globals
int VERBOSE=1,LOOP=1;
FILE *fp;

//----------------------------------------------------------------------
//  This is the Main routine for thermodl.
//
void main(short argc, char **argv)
{
   int Fahrenheit=FALSE,filenum,i,msgcnt;
   float channel[4];
   ulong st;
   char msg[255];

   // check arguments to see if request instruction with '?' or too many
   if ((argc > 4) || ((argc > 1) && (argv[1][0] == '?' || argv[1][1] == '?')))
   {
       printf("\nusage: owatod <output_filename> </Verbose> </Loop>\n"
              "  - 1-Wire A to D read on the default\n"
              "    1-Wire Net port (use SetPort32.exe to set)\n"
              "  - <output_filename> optional output filename\n"
              "  - </V> optional show verbose output\n"
              "  - </L> optional continous debit loop\n"
              "  - press any key to stop\n"
              "  - version 1.00\n");
       exit(1);
   }

   // read the default PortNum and PortType 
   if (!TMReadDefaultPort(&PortNum,&PortType))
   {
       printf("ERROR, Could not read the default PortNum and PortType!\n");
       exit(1);
   }

   // check arguments 
   VERBOSE = FALSE;
   LOOP = FALSE;
   filenum = 0;
   for (i = 1; i < argc; i++)
   {
      if (argv[i][0] != '/')
         filenum = i;
      else if ((argv[i][1] == 'V') || (argv[i][1] == 'v'))
         VERBOSE = TRUE;
      else if ((argv[i][1] == 'l') || (argv[i][1] == 'L'))
         LOOP = TRUE;
   }

   // open the output file  
   fp = NULL;
   if (filenum > 0)
   {
      fp = fopen(argv[filenum],"wb");
      if(fp == NULL)
      {    
         printf("ERROR, Could not open output file!\n");
         exit(1);
      }
      else
         printf("File '%s' opened to write sensor data.\n\n",
                 argv[filenum]);
   }

   // print the header info 
   PrintHeader("OwAtoD Version 1.00");

   // verify 1-Wire, find device, setup device
   if (!RunScript(SetupAtoD, &ROM[0], sizeof(ROM)))
   { 
      printf("Could not find 1-Wire A to D (DS2450), end program\n");
      exit(1);
   }

   // attempt to open session on 1-Wire net
   if (!RunScript(OpenSession, NULL, 0))
   { 
      printf("Could not open session on 1-Wire net, end program\n");
      exit(1);
   }
   
   st = GetTickCount();

   // read A to D with optional loop
   do
   {
      // verify 1-Wire, find device, setup device
      if (RunScript(ReadAtoD, &channel[0], sizeof(channel)))
      {
         // display results
         msgcnt = sprintf(msg,"%8ld, ", GetTickCount() - st);
         for (i = 0; i < 4; i++)
            msgcnt += sprintf(msg + msgcnt,"%6.2f%c ",channel[i], 
                      (i == 3) ? ' ' : ',');
         msgcnt += sprintf(msg + msgcnt,"\n");
      }
      else
         msgcnt = sprintf(msg,"Script did not complete\n");

      output_status(LV_ALWAYS,msg);
   }
   while (LOOP && !kbhit());

   // absorb any key presses
   while (kbhit())
      getch();

   // close session
   if (!RunScript(CloseSession, NULL, 0))
      printf("Could not close session on 1-Wire net\n");

   // close opened file
   if ((fp != NULL) && (fp != stdout))
   {
      printf("File '%s' closed.\n",
              argv[filenum]);
      fclose(fp);
   }

   printf("End program normally\n");
   exit(0);
}


//--------------------------------------------------------------------------
// Prints a message, and the current driver versions.  Return TRUE
// driver version were found and FALSE if not.
//
int PrintHeader(char *title)
{
   char tstr[80], msg[512]; 
   int msgcnt = 0,rt=FALSE;

   // print a header with driver version info   
   sprintf(msg,"%s\n",title);
   output_status(LV_ALWAYS,msg);
   output_status(LV_ALWAYS,"Dallas Semiconductor Corporation\n");
   output_status(LV_ALWAYS,"Copyright (C) 1992-1999\n\n");
   sprintf(msg,"Port number: %d    Port type: %d\n",PortNum,PortType);
   output_status(LV_ALWAYS,msg);
   Get_Version(tstr);
   sprintf(msg,"Main driver version: %s\n",tstr);
   output_status(LV_ALWAYS,msg);
   msgcnt = sprintf(msg + msgcnt,"Sub-driver version, Type%d:",PortType);
   if (TMGetTypeVersion(PortType,tstr) < 0)
      msgcnt += sprintf(msg + msgcnt,"No Hardware Driver for this type found!");
   else
   {
      rt = TRUE;
      msgcnt += sprintf(msg + msgcnt," %s\n\n\n",tstr);
   }

   // instructions
   msgcnt += sprintf(msg + msgcnt,"Press any key to stop this program.\n\n"
                     "Output [Time (ms), Channel A, Channel B, Channel C, Channel D]\n\n");

   // send output
   output_status(LV_ALWAYS,msg);

   return rt;
}


//--------------------------------------------------------------------------
//  output status message
//
void output_status(int level, char *st)
{
   // skip null strings
   if (st[0] == 0)
      return;

   // check if in verbose mode
   if ((level >= LV_ALWAYS) || VERBOSE)
   { 
      printf("%s",st);
      if(fp != NULL)
         fprintf(fp,"%s",st);
   }
}
