//---------------------------------------------------------------------------
// Copyright (C) 1999 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  owatodut.C - 1-Wire A to D Utility and script functions
//
//  Compiler:  Microsoft Visual C 
//  Version:   1.00
//  History:   
//

#include "ibtmex.h"   
#include "owatod.h"

// external function prototypes
extern int ReadFromMIB(ushort, uchar *, int);
extern void output_status(int, char *);
extern int EraseScratchSHA(void);
extern int ReadScratchSHA(ushort *, uchar *, uchar *);
extern int WriteScratchSHA(ushort, uchar *, int);
extern int CopyScratchSHA(ushort,int);
extern int ReadAuthPageSHA(ushort, uchar *);
extern int ReadPageUDP(ushort, uchar *);
extern int ComputeSHA(ushort, uchar);
extern int MatchScratchSHA(uchar *);
extern ushort docrc16(ushort);
extern int TestModeMatch(void);

// Local Function Prototypes 
int RunScript(Script script[], void *, int);
int ScriptStep(Script *, int *, int *, int *, char *, void *, int);
int SetupAtoDContol(uchar *, char *);
int WriteAtoD(uchar *, int, int);
int DoAtoDConversion(void);
int ReadAtoDResults(float *, int);
int Select(void);

// globals
extern ushort CRC16;
uchar AtoD_control[32];
int in_overdrive=0;

//--------------------------------------------------------------------------
// Script functions
//--------------------------------------------------------------------------

//--------------------------------------------------------------------------
// Run the specified script. 
// Status is printed using 'output_status' function.  The script result is
// returned in the buffer 'prslt' of max length 'rslt_max'.  Result data 
// format is script dependent.
//
// Return: TRUE - all steps completed
//         FALSE - all steps NOT completed due to error or escape 
//
int RunScript(Script script[], void *prslt, int rslt_max)
{
   char statusmsg[256],LastDescription[256],LastMsg[256];
   char tmsg[256];
   int StepCount,SubStep,ErrorCount,Status;
   uchar *puchar;   

   static ulong st = 0;

   // reset the step to the begining
   StepCount = 0;
   SubStep = 0;
   ErrorCount = 0;
   Status = STATUS_INPROGRESS;
   statusmsg[0] = 0;
   puchar = prslt;
 
   // loop to perform all of the steps to download the Thermochron
   do
   {   
      // switch on the status of the last step done
      switch(Status)
      {
         // step complete so go to the next
         case STATUS_STEP_COMPLETE:
            StepCount++;
            SubStep = 0;
            ErrorCount = 0;
            Status = STATUS_INPROGRESS;
            LastDescription[0] = 0;
            LastMsg[0] = 0;
            break;
         // in progress so call again
         case STATUS_INPROGRESS:
            // print step description if different 
            if (strcmp(LastDescription,
                script[StepCount].StepDescription) != 0)
            {
               sprintf(tmsg,"%s -->",script[StepCount].StepDescription);
               output_status(LV_VERBOSE,tmsg);
               sprintf(LastDescription,"%s",script[StepCount].StepDescription);
            }

            // perform a step in the job
            ScriptStep(&script[StepCount],&SubStep, 
                      &Status, &ErrorCount, statusmsg, 
                      prslt, rslt_max);
         
            // print results if different
            if (strcmp(LastMsg,statusmsg) != 0)
            {
               sprintf(tmsg,"%s\n",statusmsg);
               output_status(LV_VERBOSE,tmsg);
               sprintf(LastMsg,"%s",statusmsg);
            }
            else
               output_status(LV_VERBOSE,".");
            break;
         // encountered a transient error
         case STATUS_ERROR_TRANSIENT:
            // if 20 tansient errors in a row then abort
            if (ErrorCount > 20)
               Status = STATUS_ERROR_HALT;
            else
               Status = STATUS_INPROGRESS;
            break;
         // all steps complete
         case STATUS_COMPLETE:
            output_status(LV_VERBOSE,"End script normally\n");
            return TRUE;
         // non-recoverable error
         case STATUS_ERROR_HALT:
            // try to do emergency clean up
            output_status(LV_VERBOSE,"Aborting script due to non-recoverable error\n");
            return FALSE;
      }
   }
   while (!kbhit());

   // key abort
   output_status(LV_ALWAYS,"Aborting script due to key press\n");
   return FALSE;
}

//----------------------------------------------------------------------
//  Use the script to perform a step and return.
//
//  Return:
//          STATUS_STEP_COMPLETE - step complete, go to next step
//          STATUS_INPROGRESS - step is not complete, call again
//          STATUS_ERROR_HALT -  error, halt script
//          STATUS_ERROR_TRANSIENT - transient error, call again
//
int ScriptStep(Script *StateScript, int *SubStep,
              int *Status, int *ErrorCount, char *statusmsg,
              void *prslt, int rslt_max)
{
   short rslt,statuslen,j;
   short *pshort;
 
   // do the current step and return
   switch (StateScript->Step)
   {
      // attempt to get a session on the 1-Wire Net
      case ST_GET_SESSION:
         session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
         if (session_handle > 0)
         {
            sprintf(statusmsg,"Session aquired");
            *Status = STATUS_STEP_COMPLETE;
         }
         else if (session_handle == 0)
         {
            sprintf(statusmsg,"1-Wire Net in use by other process");
            *Status = STATUS_INPROGRESS;
         }
         else if (session_handle < 0)
         {
            sprintf(statusmsg,"1-Wire Net not available");
            *Status = STATUS_ERROR_HALT;
         }
         break;
      
      // perform TMSetup on the 1-Wire Net port   
      case ST_SETUP:
         rslt = TMSetup(session_handle);
         if (rslt == 1)
         {
            sprintf(statusmsg,"1-Wire setup is success");
            *Status = STATUS_STEP_COMPLETE;
            TMAutoOverDrive(session_handle,&state_buffer[0],1);
         }
         else if (rslt == 2)
         {
            sprintf(statusmsg,"1-Wire shorted");
            // check for error last time 
            *ErrorCount = *ErrorCount + 1;
            *Status = STATUS_ERROR_TRANSIENT;
         }
         else
         {
            sprintf(statusmsg,"1-Wire Net has a hardware problem, abort");
            *Status = STATUS_ERROR_HALT;
         }
         break;

      // release the session on the 1-Wire Net
      case ST_REL_SESSION:
         TMEndSession(session_handle);
         sprintf(statusmsg,"1-Wire Net released");
         *Status = STATUS_STEP_COMPLETE;
         break;      

      // the operation is complete      
      case ST_FINISH:
         sprintf(statusmsg,"Operation complete");
         *Status = STATUS_COMPLETE;
         break; 

      // find DS2450
      case ST_FIND_ATOD:
         // set up to find the first device with family 'family' 
         if (TMFamilySearchSetup(session_handle,&state_buffer[0],ATOD_FAM) == 1)
         { 
            // get first device in list with the specified family 
            if (TMNext(session_handle,&state_buffer[0]) == 1)
            {
               // check for enough memory to read ROM
               if (rslt_max >= 16)
               {
                  // read the rom number 
                  pshort = prslt;
                  pshort[0] = 0;
                  TMRom(session_handle,&state_buffer[0],pshort);   
      
                  // check if correct type 
                  if (ATOD_FAM == (pshort[0] & 0x7F))
                  {
                     statuslen = sprintf(statusmsg,"SHA iButton found: ");
                     for (j = 7; j >= 0; j--)
                        statuslen += sprintf(&statusmsg[statuslen],"%02X",pshort[j]);
                     *Status = STATUS_STEP_COMPLETE;
                     break;
                  }
               }
               else
               {
                  sprintf(statusmsg,"insufficient memory to read ROM number");
                  *Status = STATUS_ERROR_HALT;
                  break;
               }
            }
         }
         else
         {
            *Status = STATUS_ERROR_HALT;
            sprintf(statusmsg,"Lost session to 1-Wire Net");
            break;
         }
         sprintf(statusmsg,"SHA iButton not on 1-Wire Net");
         *Status = STATUS_ERROR_HALT;
         break;

      // get the setup information 
      case ST_GET_SETUP:
         if (SetupAtoDContol(AtoD_control, statusmsg))
            *Status = STATUS_STEP_COMPLETE;
         else
         {
            *Status = STATUS_ERROR_HALT;
            sprintf(statusmsg,"Could not get setup information");
         }   
         break;

      // setup the A to D
      case ST_WRITE_SETUP:
         if (WriteAtoD(AtoD_control, 0x08, 0x17))
         {
            sprintf(statusmsg,"A to D write successful");
            *Status = STATUS_STEP_COMPLETE;
         }
         else
         {
            sprintf(statusmsg,"Could setup A to D, device not present");
            // check for error last time 
            *ErrorCount = *ErrorCount + 1;
            *Status = STATUS_ERROR_TRANSIENT;
         }
         break;

      // perform an A to D conversion
      case ST_DO_CONV:
         if (DoAtoDConversion())
         {
            sprintf(statusmsg,"A to D conversion successful");
            *Status = STATUS_STEP_COMPLETE;
         }
         else
         {
            sprintf(statusmsg,"Could not do conversion, device not present");
            // check for error last time 
            *ErrorCount = *ErrorCount + 1;
            *Status = STATUS_ERROR_TRANSIENT;
         }
         break;

      // Read A to D results
      case ST_READ_RSLTS:
         if (ReadAtoDResults(prslt, rslt_max))
         {
            sprintf(statusmsg,"A to D conversion successful");
            *Status = STATUS_STEP_COMPLETE;
         }
         else
         {
            sprintf(statusmsg,"Could not do conversion, device not present");
            // check for error last time 
            *ErrorCount = *ErrorCount + 1;
            *Status = STATUS_ERROR_TRANSIENT;
         }
         break;

      default:
         *Status = STATUS_ERROR_HALT;
         sprintf(statusmsg,"1-Wire Step not known\n");
         break;
   }

   return *Status;
}


//--------------------------------------------------------------------------
// Attempt to do an A to D conversion 
//
// Returns: TRUE, success, conversion ok
//          FALSE, failure to do conversion
//
int DoAtoDConversion(void)
{
   uchar send_block[50];
   int i;
   short send_cnt=0;

   // access the device 
   if (Select() == 1)
   {
      CRC16 = 0;
      // create a block to send 
      send_block[send_cnt] = 0x3C;
      docrc16(send_block[send_cnt++]);

      // input select mask (all channels)
      send_block[send_cnt] = 0x0F;
      docrc16(send_block[send_cnt++]);

      // read-out control 
      send_block[send_cnt] = 0x00;
      docrc16(send_block[send_cnt++]);

      // read CRC16
      send_block[send_cnt++] = 0xFF;
      send_block[send_cnt++] = 0xFF;

      // now send the block
      if (TMBlockStream(session_handle,send_block,send_cnt) == send_cnt)
      {
         // check the CRC
         for (i = send_cnt - 2; i < send_cnt; i++)
            docrc16(send_block[i]);
            
         // verify CRC16 is correct
         if (CRC16 != 0xB001)
            return FALSE;

         // if success then apply the strong pullup 
         TMOneWireLevel(session_handle,LEVEL_SET,LEVEL_STRONG_PULLUP,PRIMED_NONE);
         // delay the max time, 6ms 
         Sleep(6);
         // set the pullup back to normal 
         TMOneWireLevel(session_handle,LEVEL_SET,LEVEL_NORMAL,PRIMED_NONE);
         // check conversion over 
         if (TMTouchByte(session_handle,0xFF) == 0xFF) 
            return TRUE;
      }
   }

   return FALSE;
}


//--------------------------------------------------------------------------
// Read A to D results 
//
// Returns: TRUE, success, results read ok
//          FALSE, failure to read
//
int ReadAtoDResults(float *prslt, int rslt_max)
{
   uchar read_block[50];
   int i;
   short rslt;
   ulong templong;

   // make sure result array is big enough
   if ((sizeof(prslt[0]) * 4) > rslt_max)
      return FALSE;

   // read result page
   rslt = TMExtendedReadPage(session_handle,&state_buffer[0],0,read_block,0);

   // verify read ok
   if (rslt == 8)
   {
      // convert the value read to floats
      for (i = 0; i < 4; i++)
      {   
         templong = ((read_block[i + 1] << 8) | read_block[i]) & 0x0000FFFF; 
         prslt[i] = (float)((float)(templong / 65535.0) * 
                    ((AtoD_control[i * 2 + 1] & 0x01) ? 5.12 : 2.56));
      }

      return TRUE;
   }

   return FALSE;
}

//--------------------------------------------------------------------------
// Setup A to D control data.  This is hardcoded to 5.12Volt scale at 
// 8 bits but it could be read from a file
//
int SetupAtoDContol(uchar *ctrl, char *msg)
{
   int i;

   // setup CONTROL register
   for (i = 0; i < 8; i += 2)
   {
      ctrl[i] = NO_OUTPUT | BITS_8;
      ctrl[i + 1] = RANGE_512 | ALARM_HIGH_DISABLE | ALARM_LOW_DISABLE;
   }
   // setup ALARM register
   for (i = 8; i < 16; i++)
      ctrl[i] = 0;

   // set return value
   sprintf(msg,"All channels set to 5.12V range at 8 bits");

   return TRUE;
}

//--------------------------------------------------------------------------
// Write A to D with provided buffer. 
//
// Returns: TRUE, success, results read ok
//          FALSE, failure to read
//
int WriteAtoD(uchar *ctrl, int start_address, int end_address)
{
   uchar send_block[50];
   int i;
   short send_cnt=0,flag,address;

   // build the first part of the packet to write  
   // reset CRC 
   CRC16 = 0;
   send_cnt = 0;
   // write memory command 
   send_block[send_cnt++] = 0x55;
   docrc16(0x55); 
   // address 
   send_block[send_cnt] = (uchar)(start_address & 0xFF);
   docrc16(send_block[send_cnt++]);
   send_block[send_cnt] = (uchar)((start_address >> 8) & 0xFF);
   docrc16(send_block[send_cnt++]); 

   // select the device 
   if (Select() == 1)
   {
      // loop to write each byte 
      for (address = start_address; address <= end_address; address++)
      {
         // build the packet to write 
         // init CRC on all but first pass 
         if (address != start_address) 
            CRC16 = address;
         // data to write 
         send_block[send_cnt] = ctrl[address - start_address];
         docrc16(send_block[send_cnt++]);
         // read CRC16 
         send_block[send_cnt++] = 0xFF;
         send_block[send_cnt++] = 0xFF;
         // echo of byte 
         send_block[send_cnt++] = 0xFF;

         // send the block 
         flag = TMBlockStream(session_handle, send_block, send_cnt);
         // check results of block 
         if (flag == send_cnt) 
         {
            // perform crc16 on last 2 bytes
            for (i = (send_cnt - 3); i <= (send_cnt - 2); i++)
               docrc16(send_block[i]);

            // verify crc16 is correct 
            if ((CRC16 != 0xB001) ||
                (send_block[send_cnt-1] != ctrl[address - start_address])) 
               return FALSE;
         }
         else
            return FALSE;

         // reset the packet 
         CRC16 = 0;
         send_cnt = 0;
      }
   }
   // no device 
   else
      return FALSE;

   return TRUE;
}


//--------------------------------------------------------------------------
// Select the current device and attempt overdrive if possible.
//
int Select(void)
{
   int rt;

   // select device
   rt = TMAccess(session_handle,&state_buffer[0]);

   // check if in overdrive
   in_overdrive = (TMOneWireCom(session_handle,1,0) == 1);
   
   return rt;
}


//--------------------------------------------------------------------------
// Calculate a new CRC16 from the input data shorteger.  Return the current
// CRC16 and also update the global variable CRC16.
//
static short oddparity[16] = { 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0 };

ushort docrc16(ushort data)
{
   data = (data ^ (CRC16 & 0xff)) & 0xff;
   CRC16 >>= 8;

   if (oddparity[data & 0xf] ^ oddparity[data >> 4])
     CRC16 ^= 0xc001;

   data <<= 6;
   CRC16   ^= data;
   data <<= 1;
   CRC16   ^= data;

   return CRC16;
}


